# -*- coding: utf-8 -*-

from zope.interface import implements
from Products.CMFCore.utils import getToolByName
from collective.datagridcolumns.interfaces import ICallingContextProvider

class SimpleCallingContextProvider(object):
    """
    See ICallingContextProvider docstring.
    This is a simple implementation: always call on the site root
    """
    implements(ICallingContextProvider)
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
    
    @property
    def url(self):
        return getToolByName(self.context, 'portal_url')()




# -*- coding: utf-8 -*-

from zope.component import getUtility

from zope.component.interfaces import ComponentLookupError
from zope.component import getMultiAdapter
from zope import schema

from AccessControl import ClassSecurityInfo
from Acquisition import aq_parent
from Globals import InitializeClass
from Products.CMFCore.utils import getToolByName

from Products.Archetypes import atapi

from Products.DataGridField.Column import Column

from collective.datagridcolumns.interfaces import ICallingContextProvider

class ReferenceColumn(Column):
    """ Defines column with that will contains an UID, that reference another content """

    security = ClassSecurityInfo()

    def __init__(self, label, default=None, object_provides=[], surf_site=True, search_site=True):
        Column.__init__(self, label, default=default)
        self.object_provides = object_provides
        self.surf_site = surf_site
        self.search_site = search_site

    security.declarePublic('getAJAXCallingContext')
    def getAJAXCallingContext(self, context):
        return getMultiAdapter((context, context.REQUEST), ICallingContextProvider).url
    
    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_reference_cell"

    security.declarePublic('getTarget')
    def getTarget(self, context, cell_value):
        """ Return the object that the saved UID refers to """
        if cell_value:
            reference_catalog = getToolByName(context, 'reference_catalog')
            return reference_catalog.lookupObject(cell_value)

    security.declarePublic('getAllowedInterfaces')
    def getAllowedInterfaces(self):
        """ Return the a proper comma separated strings of object_provides values """
        return ','.join(self.object_provides)

    security.declarePublic('surfSite')
    def surfSite(self):
        return self.surf_site

    security.declarePublic('searchSite')
    def searchSite(self):
        return self.search_site


InitializeClass(ReferenceColumn)



# -*- coding: utf-8 -*-

from zope.component import getUtility
from zope.component.interfaces import ComponentLookupError
from zope import schema

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass

from Products.Archetypes import atapi

from Products.DataGridField.Column import Column
from Products.DataGridField.SelectColumn import SelectColumn as BaseSelectColumn

class SelectColumn(BaseSelectColumn):
    """ Defines column with dropdown menu cells in DataGridField """

    security = ClassSecurityInfo()

    def __init__(self, label, vocabulary=None, vocabulary_factory=None, default=None):
        """ Create a SelectColumn

        @param vocabulary Vocabulary method name. This method is called
               from Archetypes instance to get values for dropdown list.
        """
        # do not call base SelectColumn constructor
        Column.__init__(self, label, default=default)
        self.vocabulary = vocabulary
        self.vocabulary_factory = vocabulary_factory


    security.declarePublic('getVocabulary')
    def getVocabulary(self, instance):
        """ Gets this column vocabulary for specific Archetypes instance
        """
        if self.vocabulary_factory:
            try:
                factory = getUtility(schema.interfaces.IVocabularyFactory, name=self.vocabulary_factory)
            except ComponentLookupError:
                raise ComponentLookupError, "Cannot find a vocabulary utility named %s" % self.vocabulary_factory
            factory_context = instance
            return atapi.DisplayList([(t.value, t.title or t.token) for t in factory(factory_context)])
        if self.vocabulary:
            try:
                func = getattr(instance, self.vocabulary)
            except AttributeError:
                raise AttributeError, "Class %s is missing vocabulary function %s" % (str(instance), self.vocabulary)
            return func()
        return atapi.DisplayList([('', '???')])

# Initializes class security
InitializeClass(SelectColumn)



# -*- coding: utf-8 -*-

# Zope imports
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass

from Products.DataGridField.Column import Column

class TextAreaColumn(Column):
    """ Textarea column """

    security = ClassSecurityInfo()

    def __init__(self, label, default=None, default_method=None, label_msgid=None,
                       rows=3, cols=0):
        Column.__init__(self, label, default, default_method, label_msgid)
        self.rows = rows
        self.cols = cols
        
    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_textarea_cell"


# Initializes class security
InitializeClass(TextAreaColumn)



# -*- coding: utf-8 -*-

from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView

try:
    import json
except ImportError:
    import simplejson as json

class DataGridAutocompleteView(BrowserView):
    """Return a search on the site for referenced objects
    
    It commonly return a JSON with results
    
    The view need a "term" parameter
    
    An optional "origin" parameter can give to the view the traversal path of the calling context
    
    """
    
    def __call__(self, *args, **kw):
        context = self.context
        request = self.request
        response = request.response
        catalog = getToolByName(context, 'portal_catalog')

        response.setHeader('Content-Type','application/json')
        response.addHeader("Cache-Control", "no-cache")
        response.addHeader("Pragma", "no-cache")

        term = request.get('term', '').lstrip()
        object_provides = request.get('object_provides')
        search_site = request.get('search_site')
        surf_site = request.get('surf_site')
        
        # PATH search
        if term.startswith('/') and surf_site:
            portal = getToolByName(context, 'portal_url').getPortalObject()
            terms = term.split('/')
            searchPath = '/' + portal.getId() + '/'.join(terms[:-1])
            lastTerm = terms[-1]
            results = catalog(path={'query': searchPath, 'depth': 1},
                              sort_on='getObjPositionInParent')
            return json.dumps([{'label': x.Title,
                                'value': x.UID,
                                'path': x.getPath()} for x in results if \
                        x.getId.startswith(lastTerm) or x.Title.startswith(lastTerm)
                    ])
        # FULLTEXT search
        elif term and search_site: # term must be something
            terms = term.strip().split()
            path = '/'.join(context.getPhysicalPath())
            kwargs = {'Title': ' AND '.join(["%s*" % x for x in terms]),
                      'sort_on': 'sortable_title',
                      'sort_order': 'reverse',
                      'path': path}
            if object_provides:
                kwargs['object_provides'] = object_provides
            results = catalog(**kwargs)
            return json.dumps([{'label': x.Title,
                                'value': x.UID,
                                'path': x.getPath()} for x in results])
        return json.dumps([])
        


# -*- coding: utf-8 -*-

from zope.interface import Interface
from zope.interface import Attribute

class ICallingContextProvider(Interface):
    """An object able to provide and URL for make a query call on the server"""

    url = Attribute("A proper site URL, where make an AJAX call")
